==> useful-1.20.7.jar: com/timezynk/useful/rest.clj <==
(ns com.timezynk.useful.rest
  (:require
   [cheshire.core :as json]
   [clojure.core.reducers :as r]
   [clojure.java.io :as io]
   [clojure.string :as s]
   [com.timezynk.useful.base64 :as b64]
   [com.timezynk.useful.date :as date]
   [com.timezynk.useful.hash :as uh]
   [slingshot.slingshot :refer [throw+]])
  (:import
   [java.net URLEncoder]))

(defn error [status message]
  (throw+ {:code status :error message}))

(defn unauthorized []
  (error 401 "Unauthorized"))

(defn not-found []
  (error 404 "Not found"))

(defn json-response [body & {:keys [status headers] :or {status 200}}]
  (if body
    {:status status
     :headers
     (merge
      {"Content-Type" "application/json"
       "Expires" "-1"
       "Cache-Control" "no-cache"}
      headers)
     :body body}
    (not-found)))

(def PLUS #"\+")

(defn encode-filename [filename]
  (-> filename
      (URLEncoder/encode "UTF-8")
      (s/replace PLUS "%20")))

(defn file-response [body & {:keys [filename content-type content-description]}]
  (if body
    {:status 200
     :headers
     {"Content-Disposition"
      (str "attachment; filename*=UTF-8''" (encode-filename filename))
      "Content-Type" content-type
      "Content-Description" content-description}
     :body body}
    (not-found)))

(defn etag-response [req response]
  (if response
    (let [if-none-match (get-in req [:headers "if-none-match"])
          ^String response (if (string? response) response (json/encode response))
          body (.getBytes response "UTF-8")
          etag (str "\"" (b64/base64-str (uh/sha256 body)) "\"")
          headers {"ETag" etag
                   "Cache-Control" "max-age=0,private"
                   "Expires" (date/to-rfc-1123 (date/now))
                   "Content-Type" "application/json; charset=utf-8"}]
      (if (and if-none-match etag (= if-none-match etag))
        {:status 304
         :body ""
         :headers headers}
        {:status 200
         :body (io/input-stream body)
         :headers (assoc headers "Content-Length" (count body))}))
    (not-found)))

(defn agent-info [req]
  (let [headers (:headers req)]
    {:user-agent (or (get headers :user-agent) (get headers "user-agent"))
     :ip (or (get headers :x-forwarded-for) (get headers "x-forwarded-for") (:remote-addr req))}))

(defn integer-param [params k default]
  (if-let [v (get params k)]
    (Long/parseLong v)
    default))

(defn remove-nil [u]
  (r/reduce
   (fn [acc k v]
     (if v
       (assoc acc k v)
       acc))
   {} u))

==> useful-4.22.0.jar: com/timezynk/useful/rest.clj <==
(ns com.timezynk.useful.rest
  (:require
   [cheshire.core :as json]
   [cheshire.generate :as gen]
   [clojure.core.reducers :as r]
   [clojure.java.io :as io]
   [clojure.string :as s]
   [com.timezynk.useful.base64 :as b64]
   [com.timezynk.useful.date :as date]
   [com.timezynk.useful.hash :as uh]
   [slingshot.slingshot :refer [throw+]])
  (:import
   [java.net URLEncoder]))

(extend-protocol gen/JSONable
  java.time.ZonedDateTime
  (to-json [zdt jg]
    (.writeString jg (.toString zdt)))

  java.time.LocalDateTime
  (to-json [dt jg]
    (.writeString jg (.toString dt)))

  java.time.LocalDate
  (to-json [d jg]
    (.writeString jg (.toString d)))

  java.time.LocalTime
  (to-json [t jg]
    (.writeString jg (.toString t))))

(defn error [status message]
  (throw+ {:code status :error message}))

(defn unauthorized []
  (error 401 "Unauthorized"))

(defn not-found []
  (error 404 "Not found"))

(defn json-response [body & {:keys [status headers] :or {status 200}}]
  (if body
    {:status status
     :headers
     (merge
      {"Content-Type" "application/json"
       "Expires" "-1"
       "Cache-Control" "no-cache"}
      headers)
     :body body}
    (not-found)))

(def PLUS #"\+")

(defn encode-filename [filename]
  (-> filename
      (URLEncoder/encode "UTF-8")
      (s/replace PLUS "%20")))

(defn file-response [body & {:keys [filename content-type content-description]}]
  (if body
    {:status 200
     :headers
     {"Content-Disposition"
      (str "attachment; filename*=UTF-8''" (encode-filename filename))
      "Content-Type" content-type
      "Content-Description" content-description}
     :body body}
    (not-found)))

(defn etag-response [req response]
  (if response
    (let [if-none-match (get-in req [:headers "if-none-match"])
          ^String response (if (string? response) response (json/encode response))
          body (.getBytes response "UTF-8")
          etag (str "\"" (b64/base64-str (uh/sha256 body)) "\"")
          headers {"ETag" etag
                   "Cache-Control" "max-age=0,private"
                   "Expires" (date/to-rfc-1123 (date/now))
                   "Content-Type" "application/json; charset=utf-8"}]
      (if (and if-none-match etag (= if-none-match etag))
        {:status 304
         :body ""
         :headers headers}
        {:status 200
         :body (io/input-stream body)
         :headers (assoc headers "Content-Length" (count body))}))
    (not-found)))

(defn agent-info [req]
  (let [headers (:headers req)]
    {:user-agent (or (get headers :user-agent) (get headers "user-agent"))
     :ip (or (get headers :x-forwarded-for) (get headers "x-forwarded-for") (:remote-addr req))}))

(defn integer-param [params k default]
  (if-let [v (get params k)]
    (Long/parseLong v)
    default))

(defn remove-nil [u]
  (r/reduce
   (fn [acc k v]
     (if v
       (assoc acc k v)
       acc))
   {} u))
